use serde::{Deserialize, Serialize};
use std::path::{PathBuf, Path};
use std::fs;
use std::io::Write;

#[derive(Serialize, Deserialize, Debug, Clone)]
struct AppConf {
    debug: bool,
    priority: usize,
    key: String,
}

impl Default for AppConf {
    fn default() -> Self {
        Self {
            debug: false,
            priority: 1,
            key: String::from("whatever"),
        }
    }
}

impl AppConf {
    fn from_file(path: &Path) -> Result<Self, Box<dyn std::error::Error>> {
        if !&path.exists() {
            return Err("Conf file not found".into());
        }
        let conf_content = fs::read_to_string(path)?;
        let res: AppConf = serde_yaml::from_str(&conf_content)?;
        return Ok(res);
    }

    fn to_file(&self, path: &Path) -> Result<(), Box<dyn std::error::Error>> {
        let mut parent = PathBuf::from(path);
        let _ = &parent.pop();
        if !&parent.exists() {
            return Err("Parent folder of conf file not found".into());
        }
        let serialized = serde_yaml::to_string(&self.clone())?;
        let mut file = fs::OpenOptions::new()
            .write(true)
            .append(false)
            .create(true)
            .open(&path)?;
        file.write_all(format!("---\n{}", serialized).as_bytes())?;
        return Ok(());
    }
}


fn main() {
    let myconf = AppConf::default();
    myconf.to_file(&Path::new("./settings.yml"));
    //myconf == Ok(AppConf)
    let myconf = AppConf::from_file(Path::new("./settings.yml"));//.unwrap()
    dbg!(myconf);
}
